' This program is provided as Freeware for personal, educational
' and non-commercial use under the modified BSD license. 
'
' Copyright (c) 2008, Oak Micros
' All rights reserved.
'
' Redistribution and use in source and binary forms, with or without 
' modification, are permitted provided that the following conditions 
' are met:
'
' * Redistributions of source code must retain the above copyright notice,
'   this list of conditions and the following disclaimer.
' * Redistributions in binary form must reproduce the above copyright notice,
'   this list of conditions and the following disclaimer in the documentation
'   and/or other materials provided with the distribution.
' * Neither the name of Oak Micros nor the names of its contributors 
'   may be used to endorse or promote products derived from this software 
'   without specific prior written permission.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
' IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
' ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
' LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR C
' ONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
' SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
' INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
' CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
' ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'

Module SoundBuzzer
' The purpose of this module is to give an example of how to use a separate
' task to asynchronous perform an action - in case turn on a buzzer.
'
' Public methods are provided to initialize and sound the buzzer without waiting.
'
' This module uses a boolean semaphore to know when the buzzer should be turned
' on. If the tasking is sleeping, then it is "woken up" immediately to start 
' sounding the buzzer and then returns to the caller. At a later time the buzzer 
' is then turned off'
'
' Debug statements are added to show the different task states and state transitions:	
' 	- sleeping (or transition to sleeping)
'   - being woken up to sound the buzzer (transition to sounding) 
'   - sounding the buzzer 
'   - asked to sound the buzzer when already awake

' task stack - made as small as possible.
Private taskStack(34) as Byte

' sempahore that controls all the action
Private sem as Boolean


' ******************************************************
' InitBuzzer()
'
' Starts buzzer task and turns off the buzzer
'
Public Sub InitBuzzer()
	Call PutPin(Pin.RedLED, 1)
	sem = False
	CallTask "BuzzerTask", taskStack
End Sub


' ******************************************************
' SoundBuzzerNoWait
'
' Turns on the buzzer immediately and returns. If the buzzer is 
' aleady on then no action it taken (even if it is about to turn off)
'
Public Sub SoundBuzzerNoWait()
	' check if buzzer already on 
	If Semaphore(sem) Then
		' resume the buzzer task from the sleep so it sounds
		Debug.Print CStr(Fmt(Timer(),1)); " Wakeup"
		ResumeTask(taskStack)
		
		' RunTask is not essential for this task and is only needed
		' for some immediate action - it should be used sparingly
		' RunTask(taskStack)
	Else
		' the buzzer is already on so do nothing
		Debug.Print CStr(Fmt(Timer(),1)); " Already awake"	
	End If
End Sub


' ******************************************************
' BuzzerTask
'
' This is the task that turns on the buzzer if requested. 
' Otherwise the task just sleeps.
'
Private Sub BuzzerTask()
	' keep going forever
	Do
		' check if the buzzer needed
		If Not Semaphore(sem) Then
			' turn on the buzzer
			Debug.Print CStr(Fmt(Timer(),1)); " Buzz"
			Call PutPin(Pin.RedLED, 0)
			' sleep and then turn off the buzzer
			Call Sleep(5.0)
			Call PutPin(Pin.RedLed, 1)
		End If
		' reset the sempahore always and sleep
		sem = False
		Debug.Print CStr(Fmt(Timer(),1)); " Sleeping"
		Call Sleep(2.0)
	Loop
End Sub